@@ -1,4 +1,4 @@
-/* $Id: davcp.c,v 1.14 2003/01/19 19:06:45 rtakano Exp $
+/* $Id: davcp.c,v 1.16 2003/01/23 04:14:39 rtakano Exp $
  * davcp: WebDAV copy
  * Copyright (c) 2002 TAKANO Ryousei <rtakano@sourceforge.jp>
  *
@@ -83,6 +83,19 @@ static int open_connection(ne_uri* uri)
   char* proxy_host;
   int proxy_port = 0;
 
+  /* Set defaults. */
+#if 0
+  if (uri->scheme == NULL) {
+    uri->scheme = ne_strndup("http", 4);
+  }
+#endif
+  if (uri->port == 0) {
+    uri->port = ne_uri_defaultport(uri->scheme);
+  }
+  if (uri->path == NULL) {
+    uri->path = "/";
+  }
+
   /* Create session. */
   session = ne_session_create(uri->scheme, uri->host, uri->port);
 
@@ -154,15 +167,12 @@ static int get_method(const char* local, const char* remote)
       }
       goto out;
     }
-  } else {
-    fprintf(stderr, "davcp: Failed: %s\n", ne_get_error(session));
-    ret = -1;
-    goto out;
   }
 
   /* get file */
   ret = stat(local, &statbuf);
-  if (ret != -1 && statbuf.st_mode & S_IFMT) {
+  if (ret != -1 && (statbuf.st_mode & S_IFMT) == S_IFDIR) {
+    /* 'local' is a directory. */
     char* p;
     p = strrchr(remote, '/');
     if (p != NULL) {
@@ -199,9 +209,6 @@ static int put_method(const char* local, const char* remote)
     {
       struct resource* reslist = NULL;
       char* col;
-      char* p;
-
-      p = strrchr(local, '/');
 
       /* check whether 'remote' is a collection? */
       col = ne_concat(remote, "/", NULL);
@@ -210,6 +217,8 @@ static int put_method(const char* local, const char* remote)
 	char* rpath;
 	struct resource* current = reslist;
 	if (current->type == resr_collection) {
+	  char* p;
+	  p = strrchr(local, '/');
 	  rpath = ne_concat(current->uri, (p ? p + 1 : local), NULL);
 	} else {
 	  rpath = strdup(current->uri);
@@ -227,7 +236,12 @@ static int put_method(const char* local, const char* remote)
   case S_IFDIR:
     {
       if (opt_recursive) {
-	ret = put_collection(session, local, remote);
+	char* p;
+	char* rpath;
+	p = strrchr(local, '/');
+	rpath = ne_concat(remote, "/", (p ? p + 1 : local), NULL);
+	ret = put_collection(session, local, rpath);
+	free(rpath);
       } else {
 	fprintf(stderr, "davcp: not recursive copy mode.\n");
 	ret = -1;
@@ -250,6 +264,8 @@ int main(int argc, char** argv)
   char* local_path;
   int direction;
   int ch;
+  int ret = 0;
+  int i;
 
   while ((ch = getopt(argc, argv, "l:nrvh")) != EOF) {
     switch(ch) {
@@ -277,59 +293,61 @@ int main(int argc, char** argv)
 
   signal(SIGINT, sig_handler);
 
-  ne_uri_parse(argv[0], &src_uri);
-  ne_uri_parse(argv[1], &dst_uri);
-  if (src_uri.scheme != NULL && dst_uri.scheme == NULL) {
-    direction = METHOD_GET;
-    uri = &src_uri;
-    local_path = argv[1];
-  } else if (src_uri.scheme == NULL && dst_uri.scheme != NULL) {
-    direction = METHOD_PUT;
-    local_path = argv[0];
-    uri = &dst_uri;
-  } else {
-    fprintf(stderr, "invalid copy direction: %s -> %s\n", argv[0], argv[1]);
-    ne_uri_free(&src_uri);
-    ne_uri_free(&dst_uri);
-    return 1;
-  }
-
-  /* Set defaults. */
-  if (uri->port == 0) {
-    uri->port = ne_uri_defaultport(uri->scheme);
-  }
-  if (uri->path == NULL) {
-    uri->path = "/";
-  }
-
   /* Initialize socket libraries */
   if (ne_sock_init()) {
     fprintf(stderr, "davcp: Failed to initialize socket libraries.\n");
-    ne_uri_free(&src_uri);
-    ne_uri_free(&dst_uri);
     return 1;
   }
 
+  /* check direction */
+  ne_uri_parse(argv[0], &src_uri);
+  ne_uri_parse(argv[argc-1], &dst_uri);
+  if (src_uri.scheme != NULL && dst_uri.scheme == NULL) {
+    direction = METHOD_GET;
+    uri = &src_uri;
+    if (argc > 2) {
+      fprintf(stderr, "davcp: too many src files.\n");
+      ret = 1;
+      goto out_free;
+    }
+  } else if (src_uri.scheme == NULL && dst_uri.scheme != NULL) {
+    direction = METHOD_PUT;
+    uri = &dst_uri;
+  } else {
+    fprintf(stderr, "davcp: invalid copy direction: %s -> %s\n", argv[0], argv[argc-1]);
+    ret = 1;
+    goto out_free;
+  }
+
   /* Initialize connection */
   open_connection(uri);
 
-  /* Dispatch a request */
-  if (ne_path_has_trailing_slash(local_path)) {
-    local_path[strlen(local_path) - 1] = '\0';
-  }
-  if (ne_path_has_trailing_slash(uri->path)) {
-    uri->path[strlen(uri->path) - 1] = '\0';
-  }
+  for (i = 0; i < argc - 1; i++) {
+    if (direction == METHOD_GET) {
+      local_path = argv[argc-1];
+    } else {
+      local_path = argv[i];
+    }
 
-  if (direction == METHOD_GET) {
-    get_method(local_path, uri->path);
-  } else if (direction == METHOD_PUT) {
-    put_method(local_path, uri->path);
+    /* Dispatch a request */
+    if (ne_path_has_trailing_slash(local_path)) {
+      local_path[strlen(local_path) - 1] = '\0';
+    }
+    if (ne_path_has_trailing_slash(uri->path)) {
+      uri->path[strlen(uri->path) - 1] = '\0';
+    }
+
+    if (direction == METHOD_GET) {
+      get_method(local_path, uri->path);
+    } else if (direction == METHOD_PUT) {
+      put_method(local_path, uri->path);
+    }
   }
 
   close_connection();
+ out_free:
   ne_uri_free(&src_uri);
   ne_uri_free(&dst_uri);
 
-  return 0;
+  return ret;
 }
