@@ -1,4 +1,4 @@
-/* $Id: davcp.h,v 1.1 2003/01/09 12:07:45 rtakano Exp $
+/* $Id: davcp.h,v 1.2 2003/01/19 19:06:45 rtakano Exp $
  * davcp: WebDAV copy
  * Copyright (c) 2002 TAKANO Ryousei <rtakano@sourceforge.jp>
  *
@@ -31,3 +31,15 @@ struct resource {
   int error_status;
   struct resource *next;
 };
+
+/* getput.c */
+extern int get_file(ne_session* session, const char* local, const char* remote);
+extern int get_collection(ne_session* session, const char* local, const char* remote);
+extern int put_file(ne_session* session, const char* local, const char* remote);
+extern int put_collection(ne_session* session, const char* local, const char* remote);
+
+/* list.c */
+extern int fetch_resource_list(ne_session *sess, const char *uri,
+			       int depth, int include_target,
+			       struct resource **reslist);
+extern void free_resource_list(struct resource *res);
